package com.himajin2001.marubatsu;

import com.himajin2001.marubatsu.control.Game;
import com.himajin2001.marubatsu.control.GameSetup;

/**
 * ○×ゲーム実行Class。
 * 
 * @author tsubasa
 */
public class MaruBatsuGame {

	private final String[] args;

	private String player1 = "";

	private String player2 = "";

	private int width = 3;

	private int hight = 3;

	private Game game = null;

	/**
	 * main()メソッド。
	 * 
	 * @param args
	 *            起動引数
	 */
	public static void main(String[] args) {
		System.out.println("====【○×ゲーム開始！！！】====");

		MaruBatsuGame main = new MaruBatsuGame(args);

		// 起動引数を確認
		System.out.println("起動引数確認:");
		if (args.length < 2) {
			System.err.println("起動引数が足りません！");
			main.showUsage();
			System.exit(1);
		}

		// 起動引数読み込み
		if (!main.readArgs()) {
			System.exit(1);
		}

		// ゲームの用意
		if (!main.setup()) {
			System.exit(1);
		}

		// ゲームの実行
		main.startGame();

		// 終了
		System.out.println("正常終了\n");
		System.exit(0);
	}

	/**
	 * コンストラクタ。
	 * 
	 * @param args
	 *            起動引数
	 */
	public MaruBatsuGame(final String[] args) {
		this.args = args;
	}

	/**
	 * Helpを表示する。
	 */
	public void showUsage() {
		System.out
				.println("Usage player1_class_name player2_class_name [board_width[ board_hight]]");
		System.out.println("        player1_class_name : FQN of Player1 class");
		System.out.println("        player2_class_name : FQN of Player2 class");
		System.out.println("        board_width : board width (default 3)");
		System.out
				.println("        board_width : board hight (default = board_width)");
	}

	/**
	 * 起動引数を解釈する
	 * 
	 * @return 正しく解釈されたらtrue。問題があったらfalse。
	 */
	public boolean readArgs() {
		this.player1 = this.args[0];
		this.player2 = this.args[1];

		System.out.println("対戦者1のClass : " + this.player1);
		System.out.println("対戦者2のClass : " + this.player2);

		// 幅が指定されているか？
		if (this.args.length >= 3) {
			String w = this.args[2];
			try {
				this.width = Integer.parseInt(w);
			} catch (NumberFormatException e) {
				System.err.println("ERROR : Width must be numeric value.");
				this.showUsage();
				return false;
			}

			if ((this.width < 3) || (this.width > 16)) {
				System.err.println("ERROR : Width must be >=3 and <= 16.");
				return false;
			}

			// デフォルトでは高さ＝幅
			this.hight = this.width;

			// 高さも指定されているか？
			if (this.args.length >= 4) {
				String h = this.args[3];
				try {
					this.hight = Integer.parseInt(h);
				} catch (NumberFormatException e) {
					System.err.println("ERROR : Hight must be numeric value.");
					this.showUsage();
					return false;
				}

				if ((this.hight < 3) || (this.hight > 16)) {
					System.err.println("ERROR : hight must be >=3 and <= 16.");
					return false;
				}
			}
		}

		System.out.println("ボードの幅 : " + this.width);
		System.out.println("ボードの高さ : " + this.hight);

		System.out.println("引数読み込み完了");

		return true;
	}

	/**
	 * ゲームの用意をする
	 * 
	 * @return 初期化成功ならtrue。
	 */
	public boolean setup() {
		GameSetup setup = new GameSetup();

		System.out.print("対戦者1の登録中…");

		// プレーヤーを登録する
		try {
			setup.addPlayer(this.player1);
		} catch (Exception e) {
			System.err.print("Could not load Player1 class " + this.player1);
			return false;
		}
		System.out.println("完了！");

		System.out.print("対戦者2の登録中…");
		try {
			setup.addPlayer(this.player2);
		} catch (Exception e) {
			System.err.print("Could not load Player2 class " + this.player1);
			return false;
		}
		System.out.println("完了！");

		// 盤面の大きさをセットする
		setup.setHight(this.hight);
		setup.setWidth(this.width);

		// ゲームを作成する
		System.out.println("対戦場の初期化中…");
		try {
			this.game = setup.getGameInstance();
		} catch (Exception e) {
			System.err.println("ゲームの初期化に失敗しました。");
			return false;
		}

		System.out.println("完了！");
		return true;
	}

	/**
	 * ゲームの開始。
	 */
	public void startGame() {
		if (this.game == null) {
			throw new IllegalStateException("ゲームの設定が完了していない。プログラムエラー。");
		}

		switch (this.game.doGame()) {
		case 1:
			System.out.println("対戦者1の勝利！！");
			break;

		case 2:
			System.out.println("対戦者2の勝利！！");
			break;

		case 0:
			System.out.println("[引き分けだったので判定に入ります。]");
			long time1 = this.game.getPlayer1time();
			long time2 = this.game.getPlayer2time();
			long turn1 = this.game.getPlayer1turn();
			long turn2 = this.game.getPlayer2turn();
			long ave1 = time1 / turn1;
			long ave2 = time2 / turn2;
			System.out.println("対戦者1の合計実行時間：" + time1 + "ナノ秒");
			System.out.println("対戦者1のターンあたりの平均実行時間：" + ave1 + "ナノ秒");
			System.out.println("対戦者2の合計実行時間：" + time2 + "ナノ秒");
			System.out.println("対戦者2のターンあたりの平均実行時間：" + ave2 + "ナノ秒");

			if (ave1 < ave2) {
				System.out.println("対戦者1の判定勝ち！！");
			} else if (ave1 > ave2) {
				System.out.println("対戦者2の判定勝ち！！");
			} else {
				System.out.println("判定でも引き分けです。");
			}
			break;

		default:
			throw new IllegalStateException("Unknown game result...");
		}
	}
}
