package com.himajin2001.marubatsu.control;

import com.himajin2001.marubatsu.board.Board;
import com.himajin2001.marubatsu.board.STONE;

/**
 * 盤面で石が縦横斜めにそろっていないかを確認するClass。
 * 
 * 継承不可。
 * 
 * @author tsubasa
 */
public final class Judge {

	/**
	 * チェック対象の盤面Object。
	 */
	private Board board = null;

	/**
	 * コンストラクタ。
	 */
	public Judge() {
		//
	}

	/**
	 * チェック対象のボードをセットする。
	 * 
	 * @param board
	 *            チェック対象の盤面Object。
	 */
	public void setBoard(final Board board) {
		if (this.board == null) {
			this.board = board;
		}
	}

	/**
	 * 盤面をチェックする。
	 * 
	 * 縦横斜めをチェックして、引数に指定された石がそろっているか判断する。
	 * 
	 * @param stone
	 *            チェック対象の石。(BLACKかWHITEであること。)
	 * @return そろっていればtrue。
	 */
	public boolean check(final STONE stone) {
		final STONE[][] hexes = this.board.getHexes();
		final int hight = this.board.getHight();
		final int width = this.board.getWidth();
		final int min = Math.min(width, hight); // 盤の短いほうの長さ

		// 縦にそろっているかのチェック
		for (int x = 0; x < width; x++) {
			int count = 0;
			for (int y = 0; y < hight; y++) {
				if (hexes[y][x].equals(stone)) {
					count = count + 1;
				}
			}
			if (count == hight) {
				// 縦にそろっている！
				return true;
			}
		}

		// 横にそろっているかのチェック
		for (int y = 0; y < hight; y++) {
			int count = 0;
			for (int x = 0; x < width; x++) {
				if (hexes[y][x].equals(stone)) {
					count = count + 1;
				}
			}
			if (count == width) {
				// 横にそろってる！
				return true;
			}
		}

		// 左上から右下への斜めのチェック
		int x = 0;
		int y = 0;
		int count = 0;
		while (x < min) {
			if (hexes[y][x].equals(stone)) {
				count = count + 1;
			}

			// 次の座標へ
			x = x + 1;
			y = y + 1;
		}

		if (count == min) {
			// 斜めにそろっている！
			return true;
		}

		// 右下から左上への斜めのチェック(正方形じゃないばあいのみ)
		if (width != hight) {
			x = width - 1;
			y = hight - 1;
			count = 0;
			while ((x >= 0) && (y >= 0)) {
				if (hexes[y][x].equals(stone)) {
					count = count + 1;
				}

				// 次の座標へ
				x = x - 1;
				y = y - 1;
			}

			if (count == min) {
				// 斜めにそろっている！
				return true;
			}
		}

		// 右上から左下への斜めのチェック
		x = width - 1;
		y = 0;
		count = 0;
		while ((x >= 0) && (y < hight)) {
			if (hexes[y][x].equals(stone)) {
				count = count + 1;
			}

			// 次の座標へ
			x = x - 1;
			y = y + 1;
		}

		if (count == min) {
			// 斜めにそろっている！
			return true;
		}

		// 左下から右上への斜めのチェック(正方形じゃないばあいのみ)
		if (width != hight) {
			x = 0;
			y = hight - 1;
			count = 0;
			while ((x < width) && (y >= 0)) {
				if (hexes[y][x].equals(stone)) {
					count = count + 1;
				}

				// 次の座標へ
				x = x + 1;
				y = y - 1;
			}

			if (count == min) {
				// 斜めにそろっている！
				return true;
			}
		}

		// どれにも当てはまっていないのでそろっていない
		return false;
	}
}
