package com.himajin2001.marubatsu.control;

import static org.junit.Assert.assertEquals;
import junitx.util.PrivateAccessor;

import org.junit.Test;

import com.himajin2001.marubatsu.board.Board;
import com.himajin2001.marubatsu.board.STONE;
import com.himajin2001.marubatsu.control.Judge;

/**
 * JudgeのTestCase
 * 
 * @author tsubasa
 */
public class TestJudge {

	/**
	 * コンストラクタのテスト {@link com.himajin2001.marubatsu.control.Judge#Judge()} のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testJudge() throws Exception {
		// Exceptionが出なければOK
		new Judge();
	}

	/**
	 * setBoard()のテスト
	 * {@link com.himajin2001.marubatsu.control.Judge#setBoard(com.himajin2001.marubatsu.board.Board)}
	 * のためのテスト・メソッド。
	 */
	@Test
	public void testSetBoard() {
		Board board = new Board();
		Judge judge = new Judge();
		judge.setBoard(board);
	}

	/**
	 * setBoard()のテスト
	 * 
	 * 2回セットしても最初のものが保持されていること。
	 * {@link com.himajin2001.marubatsu.control.Judge#setBoard(com.himajin2001.marubatsu.board.Board)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             問題があったとき。
	 */
	@Test
	public void testSetBoard2() throws Exception {
		Board board = new Board();
		Board board2 = new Board();
		Judge judge = new Judge();
		judge.setBoard(board);
		judge.setBoard(board2);

		assertEquals(board, PrivateAccessor.getField(judge, "board"));
	}

	/**
	 * 何もそろっていないケース(正方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck1() throws Exception {
		Board board = new Board();
		board.setHight(3);
		board.setWidth(3);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK },
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK },
				{ STONE.WHITE, STONE.BLACK, STONE.WHITE } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(false, judge.check(STONE.BLACK));
		assertEquals(false, judge.check(STONE.WHITE));
	}

	/**
	 * 何もそろっていないケース(縦長長方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck1_1() throws Exception {
		Board board = new Board();
		board.setHight(4);
		board.setWidth(3);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK },
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK },
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK },
				{ STONE.WHITE, STONE.BLACK, STONE.WHITE } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(false, judge.check(STONE.BLACK));
		assertEquals(false, judge.check(STONE.WHITE));
	}

	/**
	 * 何もそろっていないケース(横長長方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck1_2() throws Exception {
		Board board = new Board();
		board.setHight(3);
		board.setWidth(4);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK, STONE.WHITE },
				{ STONE.WHITE, STONE.BLACK, STONE.WHITE, STONE.BLACK } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(false, judge.check(STONE.BLACK));
		assertEquals(false, judge.check(STONE.WHITE));
	}

	/**
	 * 縦にそろっているケース (正方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck2() throws Exception {
		Board board = new Board();
		board.setHight(3);
		board.setWidth(3);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK },
				{ STONE.BLACK, STONE.WHITE, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(true, judge.check(STONE.BLACK));
		assertEquals(true, judge.check(STONE.WHITE));
	}

	/**
	 * 縦にそろっているケース (縦長長方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck2_1() throws Exception {
		Board board = new Board();
		board.setHight(4);
		board.setWidth(3);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK },
				{ STONE.BLACK, STONE.WHITE, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(true, judge.check(STONE.BLACK));
		assertEquals(true, judge.check(STONE.WHITE));
	}

	/**
	 * 縦にそろっているケース (横長長方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck2_2() throws Exception {
		Board board = new Board();
		board.setHight(3);
		board.setWidth(4);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK, STONE.BLACK },
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK, STONE.BLACK },
				{ STONE.BLACK, STONE.WHITE, STONE.WHITE, STONE.WHITE } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(true, judge.check(STONE.BLACK));
		assertEquals(true, judge.check(STONE.WHITE));
	}

	/**
	 * 横にそろっているケース(正方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck3() throws Exception {
		Board board = new Board();
		board.setHight(3);
		board.setWidth(3);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.BLACK, STONE.BLACK, STONE.BLACK },
				{ STONE.WHITE, STONE.WHITE, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(true, judge.check(STONE.BLACK));
		assertEquals(true, judge.check(STONE.WHITE));
	}

	/**
	 * 横にそろっているケース (縦長長方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck3_1() throws Exception {
		Board board = new Board();
		board.setHight(4);
		board.setWidth(3);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.BLACK, STONE.BLACK, STONE.BLACK },
				{ STONE.WHITE, STONE.WHITE, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(true, judge.check(STONE.BLACK));
		assertEquals(true, judge.check(STONE.WHITE));
	}

	/**
	 * 横にそろっているケース (横長長方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck3_2() throws Exception {
		Board board = new Board();
		board.setHight(3);
		board.setWidth(4);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.BLACK, STONE.BLACK, STONE.BLACK, STONE.BLACK },
				{ STONE.WHITE, STONE.WHITE, STONE.WHITE, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.WHITE, STONE.WHITE } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(true, judge.check(STONE.BLACK));
		assertEquals(true, judge.check(STONE.WHITE));
	}

	/**
	 * 左上→右下にそろっているケース(正方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck4() throws Exception {
		Board board = new Board();
		board.setHight(3);
		board.setWidth(3);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.BLACK, STONE.BLACK, STONE.WHITE },
				{ STONE.WHITE, STONE.BLACK, STONE.WHITE },
				{ STONE.WHITE, STONE.WHITE, STONE.BLACK } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(true, judge.check(STONE.BLACK));
		assertEquals(false, judge.check(STONE.WHITE));
	}

	/**
	 * 左上→右下にそろっているケース (縦長長方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck4_1() throws Exception {
		Board board = new Board();
		board.setHight(4);
		board.setWidth(3);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.WHITE, STONE.BLACK, STONE.BLACK },
				{ STONE.BLACK, STONE.WHITE, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(false, judge.check(STONE.BLACK));
		assertEquals(true, judge.check(STONE.WHITE));
	}

	/**
	 * 左上→右下にそろっているケース (横長長方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck4_2() throws Exception {
		Board board = new Board();
		board.setHight(3);
		board.setWidth(4);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.BLACK, STONE.BLACK, STONE.WHITE, STONE.BLACK },
				{ STONE.WHITE, STONE.BLACK, STONE.WHITE, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK, STONE.WHITE } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(true, judge.check(STONE.BLACK));
		assertEquals(false, judge.check(STONE.WHITE));
	}

	/**
	 * 右下→左上にそろっているケース (縦長長方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck4_3() throws Exception {
		Board board = new Board();
		board.setHight(4);
		board.setWidth(3);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.WHITE, STONE.BLACK, STONE.BLACK },
				{ STONE.WHITE, STONE.BLACK, STONE.WHITE },
				{ STONE.WHITE, STONE.WHITE, STONE.BLACK },
				{ STONE.BLACK, STONE.WHITE, STONE.WHITE } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(false, judge.check(STONE.BLACK));
		assertEquals(true, judge.check(STONE.WHITE));
	}

	/**
	 * 右下→左上にそろっているケース (横長長方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck4_4() throws Exception {
		Board board = new Board();
		board.setHight(3);
		board.setWidth(4);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.BLACK, STONE.BLACK, STONE.WHITE, STONE.BLACK },
				{ STONE.WHITE, STONE.BLACK, STONE.BLACK, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.WHITE, STONE.BLACK } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(true, judge.check(STONE.BLACK));
		assertEquals(false, judge.check(STONE.WHITE));
	}

	/**
	 * 右上→左下にそろっているケース(正方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck5() throws Exception {
		Board board = new Board();
		board.setHight(3);
		board.setWidth(3);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.BLACK, STONE.BLACK, STONE.WHITE },
				{ STONE.WHITE, STONE.WHITE, STONE.BLACK },
				{ STONE.WHITE, STONE.WHITE, STONE.BLACK } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(false, judge.check(STONE.BLACK));
		assertEquals(true, judge.check(STONE.WHITE));
	}

	/**
	 * 右上→左下にそろっているケース (縦長長方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck5_1() throws Exception {
		Board board = new Board();
		board.setHight(4);
		board.setWidth(3);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.WHITE, STONE.BLACK, STONE.BLACK },
				{ STONE.BLACK, STONE.BLACK, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(true, judge.check(STONE.BLACK));
		assertEquals(false, judge.check(STONE.WHITE));
	}

	/**
	 * 右上→左下にそろっているケース (横長長方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck5_2() throws Exception {
		Board board = new Board();
		board.setHight(3);
		board.setWidth(4);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.WHITE, STONE.BLACK, STONE.WHITE, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.WHITE, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK, STONE.BLACK } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(false, judge.check(STONE.BLACK));
		assertEquals(true, judge.check(STONE.WHITE));
	}

	/**
	 * 左下→右上にそろっているケース (縦長長方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck5_3() throws Exception {
		Board board = new Board();
		board.setHight(4);
		board.setWidth(3);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.WHITE, STONE.BLACK, STONE.BLACK },
				{ STONE.WHITE, STONE.WHITE, STONE.BLACK },
				{ STONE.WHITE, STONE.BLACK, STONE.BLACK },
				{ STONE.BLACK, STONE.WHITE, STONE.WHITE } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(true, judge.check(STONE.BLACK));
		assertEquals(false, judge.check(STONE.WHITE));
	}

	/**
	 * 左下→右上にそろっているケース (横長長方形)
	 * 
	 * {@link com.himajin2001.marubatsu.control.Judge#check(com.himajin2001.marubatsu.board.STONE)}
	 * のためのテスト・メソッド。
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testCheck5_4() throws Exception {
		Board board = new Board();
		board.setHight(3);
		board.setWidth(4);
		board.initBoard();

		// ダミーのデータを挿入
		PrivateAccessor.setField(board, "hexes", new STONE[][] {
				{ STONE.BLACK, STONE.WHITE, STONE.BLACK, STONE.WHITE },
				{ STONE.WHITE, STONE.BLACK, STONE.BLACK, STONE.WHITE },
				{ STONE.BLACK, STONE.WHITE, STONE.WHITE, STONE.BLACK } });
		Judge judge = new Judge();
		judge.setBoard(board);

		assertEquals(true, judge.check(STONE.BLACK));
		assertEquals(false, judge.check(STONE.WHITE));
	}
}
